//
// "$Id: CubeView.h,v 1.5 2004/10/17 20:05:06 Administrator Exp $"
//
// CubeView class definitions for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2003 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "fltk-bugs@fltk.org".
//

/*******************************************************************
*  This demo has been modified by Doanna Weissgerber
*  A trackball written by Daniel Keller has been added
*******************************************************************/

#ifndef CUBEVIEW_H
#define CUBEVIEW_H 1
#endif

#include <FL/Fl.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/gl.h>
#include <stdio.h>
#include <stdlib.h>
#include <fstream>
#include <iostream>
#include <cstdlib>
#include <string>
#include <vector>
#include <errno.h>
#include <GL/glut.h>
#include "FileWindow.h"
#include "DirColumn.h"

//this is the up vector for the eye position
#define UP_X 0.0
#define UP_Y 1.0
#define UP_Z 0.0
#define EYE_DISTANCE 10.0

//these are relative to your screen size.
#define WINDOW_SCALE 162
#define COLOFFSET 2.8

#define DIRCOL_SPACING 1.0

//this is the max number of elements that can be selected
#define PICK_BUF_SIZE 1024

//this is kind of sad. bigger number, slower animation.
//DEFINED IN MAKEFILE
//#define ANIMATION_SPEED 6.0

#define MOUSE_YSENSITIVITY 0.7
//this is the fraction of the window width the user must travel
//to move columns
#define MOUSE_XSENSITIVITY 0.15

//these are needed because FLTK does not implement them.
//DEFINED IN MAKEFILE
//#define MOUSE_SCROLL_UP 4
//#define MOUSE_SCROLL_DOWN 5


using namespace std;

class CubeView : public Fl_Gl_Window {

public:
    // this value determines the scaling factor used to draw the cube.
    double size;

    /**************************************************************
    *  Constructor
    **************************************************************/
    CubeView(int x,int y,int w,int h,const char *l=0);

    /************************************************************
     * Sets the x shift of the cube view camera.
     *
     * 	This function is called by the slider in CubeViewUI and the
     *  initialize button in CubeViewUI.
     ************************************************************/
    void panx(float x){xpan=x;};

    /************************************************************ 
     *  Sets the y shift of the cube view camera.
     *
     * This function is called by the slider in CubeViewUI and the
     * initialize button in CubeViewUI.
     ************************************************************/
    void pany(float y){ypan=y;};

    /********************************************************
     * The widget class draw() override.
     *
     *The draw() function initialize Gl for another round o f drawing
     * then calls specialized functions for drawing each of the
     * entities displayed in the cube view.
     *
     ******************************************************************/
    void draw();   

   
 
    /**************************************************************
    *  Handles FLTK events
    ***************************************************************/
    int	     handle(int event);

    /**************************************************************/
    //the rest of the GUI calls
    void load(char*);

    //changes the drawing style (solid, wireframe, points)
    void drawStyle(GLenum);
    void lineThickness(float);
    void pointSize(float);
    void heightScale(float);
    void maxColor(float, float, float);
    void minColor(float, float, float);
    void antiAlias(bool);
    void moveEye(float, float);
    void moveCircle(float);

    //these functions turn the specified light on or off
    void lightSwitch(GLenum, bool);
    void ambient(bool);

    //this flag tells if we draw the axis or not
    bool drawGuides;
    //and this one if we draw the light bulbs as points
    bool drawSources;

    //this gives the height of the spotlight above the map
    float spotHeight;

    //this is the mouse sensitivity when moving
    float sensitivity;

private:
    //this is the list of all the objects to draw.
    vector<DirColumn*> cols;

    //these are the sizes of each draw window
    //they change according to the draw window size and how many columns
    //we display at once (max 3)
    float colWidth, colHeight;

    //this is the relative x coordinate of the sphere
    GLfloat sphereX;

    //this is the height scale
    float hScale;

    //these are for mouse movement
    int windowHeight, windowWidth;
    int lastMouseY, lastMouseX, xPadBegin;

    //these are the pan values to move the object
    float xpan, ypan;

    //these are the values that change the eye position
    float eyePos[3];

    //this animates moving the columns left or right by amount.
    void strafeCols(float amount);

    //returns true if d is past the right edge of the window
    bool isPastRight(DirColumn* d);

    //these handle the selection of a object
    int setupSelection(GLdouble x, GLdouble y, GLdouble width, GLdouble height,
		       GLuint pickBuffer[]);
    void onClick(int, int x, int y); 
    int processHits(GLint, GLuint*, vector<void*>*);

    //returns the index of the column at x, y.
    //if no column is there, returns -1.
    int CubeView::getCurrentCol(int x, int y);

    //this draws the map from the points in the vector above
    void drawMap(void);

    //this draws three lights
    void renderLights(GLdouble);

    //these are colors for the lights
    GLfloat white[4];
    GLfloat black[4];
    GLfloat grey[4];
    GLfloat cyan[4];
    GLfloat yellow[4];
    GLfloat ambientColor[4];

    //this is the style to draw with (points, wireframe, solid)
    //and the other styles
    GLenum style;
    GLfloat lineWidth;
    GLfloat ptSize;



    /**********************************************************  
     *  Draw the cube boundaries
     *
     *  Draw the faces of the cube using the boxv[] vertices, using
     * GL_LINE_LOOP for the faces. The color is \#defined by CUBECOLOR.
     **********************************************************/
    void drawCube();
    
    float xshift,yshift,zshift;



    enum {
   	 X_AXIS = 0,
	    Y_AXIS = 1,
	    Z_AXIS = 2
	};

};

//
// End of "$Id: CubeView.h,v 1.5 2004/10/17 20:05:06 Administrator Exp $".
//
